==> packager-9.7.18-sources.jar: com/xebialabs/xldeploy/packager/io/Streamer.scala <==
/**
 * Copyright 2014-2019 XebiaLabs Inc. and its affiliates. Use is subject to terms of the enclosed Legal Notice.
 */
package com.xebialabs.xldeploy.packager.io

import java.io._
import java.util.{Arrays => JArrays, Enumeration => JEnumeration}

import org.apache.commons.compress.archivers.ArchiveInputStream
import org.apache.commons.compress.archivers.jar.JarArchiveInputStream
import org.apache.commons.compress.archivers.tar.TarArchiveInputStream
import org.apache.commons.compress.archivers.zip._

import scala.collection.JavaConverters._
import scala.util.Try

sealed trait Streamer {
  def stream(): Stream[StreamEntry]
}

class DirectoryStreamer(dir: File) extends Streamer {

  private[this] def fileStream(dir: File, base: String): Stream[StreamEntry] = {
    if (!dir.isDirectory) { return Stream.empty }

    Option(dir.listFiles)
      .map(a => JArrays.asList(a: _*).asScala.sortBy(_.getPath).toStream.flatMap(file =>
        if (file.isDirectory) {
          DirectoryEntry(file, base) #:: fileStream(file, base)
        } else {
          FileEntry(file, base) #:: fileStream(file, base)
        }))
      .getOrElse(Stream.empty)
  }

  override def stream(): Stream[StreamEntry] = fileStream(dir, dir.getPath + "/")
}

class EmptyStreamer() extends Streamer {
  override def stream(): Stream[StreamEntry] = Stream.empty[StreamEntry]
}

class FileStreamer(file: File) extends Streamer {
  override def stream(): Stream[StreamEntry] = FileEntry(file, file.getParentFile.getPath + "/") #:: Stream.empty[StreamEntry]
}

class FileStreamStreamer(is: InputStream, name: String) extends Streamer {
  override def stream(): Stream[StreamEntry] = FileStreamEntry(is, name) #:: Stream.empty[StreamEntry]
}

trait ArchiveEntryStreamer extends Streamer {
  def nextEntry(s: ArchiveInputStream, toClose: Set[Closeable]): Stream[StreamEntry] = {
    Option(s.getNextEntry) match {
      case None =>
        toClose.foreach(c => Try(c.close()))
        Stream.empty
      case Some(e) => ArchivedEntry(e, s) #:: nextEntry(s, toClose)
    }
  }
}

trait ZipEntryStreamer extends Streamer {
  def nextEntry(s: ZipArchiveInputStream, toClose: Set[Closeable]): Stream[StreamEntry] = {
    Option(s.getNextZipEntry) match {
      case None =>
        toClose.foreach(c => Try(c.close()))
        Stream.Empty
      case Some(e) => ZipArchivedEntry(e, s) #:: nextEntry(s, toClose)
    }
  }
}

trait ZipFileEntryStreamer extends Streamer {
  def nextEntry(zf: ZipFile, zfe: JEnumeration[ZipArchiveEntry], toClose: Set[Closeable]): Stream[StreamEntry] = {
    if (zfe.hasMoreElements) {
      val e = zfe.nextElement()
      ZipArchivedZipFileEntry(e, zf) #:: nextEntry(zf, zfe, toClose)
    } else {
      toClose.foreach(c => Try(c.close()))
      Stream.Empty
    }
  }
}

trait JarEntryStreamer extends Streamer {
  def nextEntry(s: JarArchiveInputStream, toClose: Set[Closeable]): Stream[StreamEntry] = {
    Option(s.getNextJarEntry) match {
      case None =>
        toClose.foreach(c => Try(c.close()))
        Stream.Empty
      case Some(e) => JarArchivedEntry(e, s) #:: nextEntry(s, toClose)
    }
  }
}

trait JarFileEntryStreamer extends Streamer {
  def nextEntry(zf: ZipFile, zfe: JEnumeration[ZipArchiveEntry], toClose: Set[Closeable]): Stream[StreamEntry] = {
    if (zfe.hasMoreElements) {
      val e = zfe.nextElement()
      JarArchivedZipFileEntry(e, zf) #:: nextEntry(zf, zfe, toClose)
    } else {
      toClose.foreach(c => Try(c.close()))
      Stream.Empty
    }
  }
}

class ZipFileArchiveStreamer(file: File) extends ZipFileEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val zf = new ZipFile(file)
    nextEntry(zf, zf.getEntriesInPhysicalOrder, Set(zf))
  }
}

class ZipArchiveStreamStreamer(is: InputStream) extends ZipEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val archiveStream = new ZipArchiveInputStream(is)
    nextEntry(archiveStream, Set.empty)
  }
}

class JarFileArchiveStreamer(file: File) extends JarFileEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val zf = new ZipFile(file)
    nextEntry(zf, zf.getEntriesInPhysicalOrder, Set(zf))
  }
}

class JarArchiveStreamStreamer(is: InputStream) extends JarEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val archiveStream = new JarArchiveInputStream(is)
    nextEntry(archiveStream, Set.empty)
  }
}

// it's a +de+compressor since it's an InputStream
class ArchiveStreamer(file: File, val decompressor: InputStream => InputStream) extends ArchiveEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val is = new FileInputStream(file)
    val decompressedStream = decompressor(is)
    val archiveStream = new TarArchiveInputStream(decompressedStream)
    nextEntry(archiveStream, Set(is, decompressedStream, archiveStream))
  }
}

// it's a +de+compressor since it's an InputStream
class ArchiveStreamStreamer(is: InputStream, val decompressor: InputStream => InputStream) extends ArchiveEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val decompressedStream = decompressor(is)
    val archiveStream = new TarArchiveInputStream(decompressedStream)
    nextEntry(archiveStream, Set.empty)
  }
}

final case class UnsupportedArchiveExtensionException(str: String) extends RuntimeException(str)

==> packager-9.7.21-sources.jar: com/xebialabs/xldeploy/packager/io/Streamer.scala <==
/**
 * Copyright 2014-2019 XebiaLabs Inc. and its affiliates. Use is subject to terms of the enclosed Legal Notice.
 */
package com.xebialabs.xldeploy.packager.io

import java.io._
import java.util.{Arrays => JArrays, Enumeration => JEnumeration}

import org.apache.commons.compress.archivers.ArchiveInputStream
import org.apache.commons.compress.archivers.jar.JarArchiveInputStream
import org.apache.commons.compress.archivers.tar.TarArchiveInputStream
import org.apache.commons.compress.archivers.zip._

import scala.collection.JavaConverters._
import scala.util.Try

sealed trait Streamer {
  def stream(): Stream[StreamEntry]
}

class DirectoryStreamer(dir: File) extends Streamer {

  private[this] def fileStream(dir: File, base: String): Stream[StreamEntry] = {
    if (!dir.isDirectory) { return Stream.empty }

    Option(dir.listFiles)
      .map(a => JArrays.asList(a: _*).asScala.sortBy(_.getPath).toStream.flatMap(file =>
        if (file.isDirectory) {
          DirectoryEntry(file, base) #:: fileStream(file, base)
        } else {
          FileEntry(file, base) #:: fileStream(file, base)
        }))
      .getOrElse(Stream.empty)
  }

  override def stream(): Stream[StreamEntry] = fileStream(dir, dir.getPath + "/")
}

class EmptyStreamer() extends Streamer {
  override def stream(): Stream[StreamEntry] = Stream.empty[StreamEntry]
}

class FileStreamer(file: File) extends Streamer {
  override def stream(): Stream[StreamEntry] = FileEntry(file, file.getParentFile.getPath + "/") #:: Stream.empty[StreamEntry]
}

class FileStreamStreamer(is: InputStream, name: String) extends Streamer {
  override def stream(): Stream[StreamEntry] = FileStreamEntry(is, name) #:: Stream.empty[StreamEntry]
}

trait ArchiveEntryStreamer extends Streamer {
  def nextEntry(s: ArchiveInputStream, toClose: Set[Closeable]): Stream[StreamEntry] = {
    Option(s.getNextEntry) match {
      case None =>
        toClose.foreach(c => Try(c.close()))
        Stream.empty
      case Some(e) => ArchivedEntry(e, s) #:: nextEntry(s, toClose)
    }
  }
}

trait ZipEntryStreamer extends Streamer {
  def nextEntry(s: ZipArchiveInputStream, toClose: Set[Closeable]): Stream[StreamEntry] = {
    Option(s.getNextZipEntry) match {
      case None =>
        toClose.foreach(c => Try(c.close()))
        Stream.Empty
      case Some(e) => ZipArchivedEntry(e, s) #:: nextEntry(s, toClose)
    }
  }
}

trait ZipFileEntryStreamer extends Streamer {
  def nextEntry(zf: ZipFile, zfe: JEnumeration[ZipArchiveEntry], toClose: Set[Closeable]): Stream[StreamEntry] = {
    if (zfe.hasMoreElements) {
      val e = zfe.nextElement()
      ZipArchivedZipFileEntry(e, zf) #:: nextEntry(zf, zfe, toClose)
    } else {
      toClose.foreach(c => Try(c.close()))
      Stream.Empty
    }
  }
}

trait JarEntryStreamer extends Streamer {
  def nextEntry(s: JarArchiveInputStream, toClose: Set[Closeable]): Stream[StreamEntry] = {
    Option(s.getNextJarEntry) match {
      case None =>
        toClose.foreach(c => Try(c.close()))
        Stream.Empty
      case Some(e) => JarArchivedEntry(e, s) #:: nextEntry(s, toClose)
    }
  }
}

trait JarFileEntryStreamer extends Streamer {
  def nextEntry(zf: ZipFile, zfe: JEnumeration[ZipArchiveEntry], toClose: Set[Closeable]): Stream[StreamEntry] = {
    if (zfe.hasMoreElements) {
      val e = zfe.nextElement()
      JarArchivedZipFileEntry(e, zf) #:: nextEntry(zf, zfe, toClose)
    } else {
      toClose.foreach(c => Try(c.close()))
      Stream.Empty
    }
  }
}

class ZipFileArchiveStreamer(file: File, encoding: String) extends ZipFileEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val zf = new ZipFile(file, encoding, true)
    nextEntry(zf, zf.getEntriesInPhysicalOrder, Set(zf))
  }
}

class ZipArchiveStreamStreamer(is: InputStream, encoding: String) extends ZipEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val archiveStream = new ZipArchiveInputStream(is, encoding, true)
    nextEntry(archiveStream, Set.empty)
  }
}

class JarFileArchiveStreamer(file: File, encoding: String) extends JarFileEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val zf = new ZipFile(file, encoding, true)
    nextEntry(zf, zf.getEntriesInPhysicalOrder, Set(zf))
  }
}

class JarArchiveStreamStreamer(is: InputStream, encoding: String) extends JarEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val archiveStream = new JarArchiveInputStream(is, encoding)
    nextEntry(archiveStream, Set.empty)
  }
}

// it's a +de+compressor since it's an InputStream
class ArchiveStreamer(file: File, val decompressor: InputStream => InputStream) extends ArchiveEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val is = new FileInputStream(file)
    val decompressedStream = decompressor(is)
    val archiveStream = new TarArchiveInputStream(decompressedStream)
    nextEntry(archiveStream, Set(is, decompressedStream, archiveStream))
  }
}

// it's a +de+compressor since it's an InputStream
class ArchiveStreamStreamer(is: InputStream, val decompressor: InputStream => InputStream) extends ArchiveEntryStreamer {
  override def stream(): Stream[StreamEntry] = {
    val decompressedStream = decompressor(is)
    val archiveStream = new TarArchiveInputStream(decompressedStream)
    nextEntry(archiveStream, Set.empty)
  }
}

final case class UnsupportedArchiveExtensionException(str: String) extends RuntimeException(str)
